module IR_PROCESS ( 
 input         CLOCK_50 , 
 input         RESET_N , 
 input         IR,
 output reg    TRACK ,
 output reg    PLAY,
 output reg    FD_BK	, 
 output        START,
 output        TURN_R	, 
 output        TURN_L  , 
 output        RET ,
 output        SPA_LAMP , 
 output        IR_TR ,
 output        CH_CLEAR 
) ; 

//-----IR CODE ---- 
parameter NAMUAL  = 16'hEE11; 
parameter GO      = 16'hE916; 
parameter STOPP   = 16'hF30C; 
parameter RIGHT   = 16'hE718; 
parameter LEFT    = 16'hEB14; 
parameter UP      = 16'hE41B; 
parameter DOWN    = 16'hE01F; 
parameter FORD    = 16'hE51A; 
parameter BACK    = 16'hE11E; 
parameter RETURN  = 16'hE817; 

parameter SEC_3   = 50000000*1 ; 
parameter SEC_5   = 50000000*5 ; 

//---WIRE /REG --- 
wire       START_TR ; 
reg [15:0] rM  ; 
reg        PLAY_OFF ; 
//--IR RECIEVER -- 
wire        data_ready ;
wire [31:0] hex_data;
reg  [31:0] TIMER ;  
//------
wire    CLEAR ;
assign  CLEAR  = TRACK & RESET_N ; 

//---TRACK ENABLE/DISABLE
always @( negedge RESET_N   or  negedge  data_ready ) 
 if ( !RESET_N ) begin 
    TRACK  <= 0 ;
 end 	
 else begin   
        if (hex_data [31:16] == NAMUAL ) TRACK  <=0;	
   else if (hex_data [31:16] == GO )     TRACK  <=1;
 end 	


//-- START TRACK PLAY --
always @( negedge CLEAR or posedge  CLOCK_50 )  begin 
if (!CLEAR) begin 
  TIMER <=0 ;
  PLAY  <=0; 
 
end
else  begin 
	if  ( TIMER > SEC_3 )    PLAY <=1; 
	else TIMER <= TIMER +1  ;
end
end	
		
//--- GO START ----- 
 assign START_TR = (
     (hex_data [31:16] == FORD ) || 
	  (hex_data [31:16] == BACK ) || 
	  (hex_data [31:16] == RIGHT) || 
	  (hex_data [31:16] == LEFT ) 
	  ); 
	  
 TURN_DELAY  trsy(
  .RESET_N    ( RESET_N) , 
  .CLK_50     ( CLOCK_50) , 
  .CODE_TR    ( START_TR ),
  .READY      ( IR_TR ), 
  .DELAY1_OUT ( START ) 
 );	
	
//---FORD-BK 	
 always @( negedge RESET_N  or  negedge  data_ready ) 
 if ( !RESET_N ) FD_BK <=0; 
 else if (hex_data [31:16] == FORD ) FD_BK <=0; 
 else if (hex_data [31:16] == BACK ) FD_BK <=1; 
	

//---RIGHT KEY 
TURN_DELAY  trr (
  .RESET_N    ( RESET_N  ) , 
  .CLK_50     ( CLOCK_50) , 
  .CODE_TR    ((hex_data [31:16]==RIGHT ) ), 
  .READY      ( IR_TR ), 
  .DELAY1_OUT ( TURN_R) 
);

//--LEFT KEY 
TURN_DELAY  tll(
  .RESET_N    (RESET_N ) , 
  .CLK_50     (CLOCK_50) , 
  .CODE_TR    ((hex_data [31:16]==LEFT ) ), 
  .READY      (IR_TR ), 
  .DELAY1_OUT (TURN_L) 
);
	
//--RETURN( SPEAKER - LAMP SWITCH) --- 
TURN_DELAY  trt(
  .RESET_N    (RESET_N ) , 
  .CLK_50     (CLOCK_50) , 
  .CODE_TR    ((hex_data [31:16]==RETURN ) ), 
  .READY      (data_ready), 
  .DELAY1_OUT ( SPA_LAMP ) 
);	
//-- IR DATA RECEIVER---
IR_RECEIVE u1(	
  .iCLK  ( CLOCK_50), 
  .iRST_n( RESET_N ),        
  .iIRDA ( IR), 
  .oDATA_READY ( data_ready),
  .oDATA       ( hex_data)        
);

//--IR reTRIGGER--  
IR_DATA_TR  irtr(
  .CLOCK_50 ( CLOCK_50) , 
  .RESET_N  ( RESET_N), 
  .IR       ( IR),
  .IR_LATCH ( IR_TR) 
);
 
endmodule 