
//=======================================================
//  This code is generated by Terasic System Builder
//=======================================================

module DE0_NANO_SMART_CAR(

	//////////// CLOCK //////////
	input 		          		CLOCK_50,

	//////////// LED //////////
	output		     [7:0]		LED,

	//////////// KEY //////////
	input 		     [1:0]		KEY,

	//////////// SW //////////
	input 		     [3:0]		SW,

	//////////// SDRAM //////////
	output		    [12:0]		DRAM_ADDR,
	output		     [1:0]		DRAM_BA,
	output		          		DRAM_CAS_N,
	output		          		DRAM_CKE,
	output		          		DRAM_CLK,
	output		          		DRAM_CS_N,
	inout 		    [15:0]		DRAM_DQ,
	output		     [1:0]		DRAM_DQM,
	output		          		DRAM_RAS_N,
	output		          		DRAM_WE_N,

	//////////// Accelerometer and EEPROM //////////
	output		          		G_SENSOR_CS_N,
	input 		          		G_SENSOR_INT,
	output		          		I2C_SCLK,
	inout 		          		I2C_SDAT,

	//////////// GPIO_0, GPIO_0 connect to Smart Car Daughtercard //////////
	output		          		BUZZER_OUT,
	output		          		IR_LED_ON,
	input 		          		IR_RXD,
	output		          		LED_LEFT,
	output		          		LED_RIGHT,
	output		          		LTC_ADC_CONVST,
	output		          		LTC_ADC_SCK,
	output		          		LTC_ADC_SDI,
	input 		          		LTC_ADC_SDO,
	output		          		MTRL_N,
	output		          		MTRL_P,
	output		          		MTRR_N,
	output		          		MTRR_P,
	input 		          		MTR_Fault_n,
	output		          		MTR_Sleep_n,
	inout 		     [7:0]		TMD0,
	inout 		     [7:0]		TMD1,
	output		          		VCC3P3_PWRON_n,

	//////////// GPIO_1, GPIO_1 connect to LT24 - 2.4" LCD and Touch //////////
	input 		          		LT24_ADC_BUSY,
	output		          		LT24_ADC_CS_N,
	output		          		LT24_ADC_DCLK,
	output		          		LT24_ADC_DIN,
	input 		          		LT24_ADC_DOUT,
	input 		          		LT24_ADC_PENIRQ_N,
	output		          		LT24_CS_N,
	output		    [15:0]		LT24_D,
	output		          		LT24_LCD_ON,
	output		          		LT24_RD_N,
	output		          		LT24_RESET_N,
	output		          		LT24_RS,
	output		          		LT24_WR_N
);



//=======================================================
//  REG/WIRE declarations
//=======================================================


//wire sensor_reset_n;
wire buzzer_on;
wire reset_n;



//=======================================================
//  Structural coding
//=======================================================


assign MTR_Sleep_n = 1;
assign reset_n = KEY[0] | KEY[1]; // press both key together to reset





    nios_system u0 (
        .reset_reset_n                          (reset_n),                          //                           reset.reset_n
        .clk_clk                                (CLOCK_50),                                //                             clk.clk
        .key_external_connection_export         (KEY),         //         key_external_connection.export
		  
		  
        .adc_ltc2308_conduit_end_convst            (LTC_ADC_CONVST),            //            adc_ltc2308_conduit_end.convst
        .adc_ltc2308_conduit_end_sck               (LTC_ADC_SCK),               //                                   .sck
        .adc_ltc2308_conduit_end_sdi               (LTC_ADC_SDI),               //                                   .sdi
        .adc_ltc2308_conduit_end_sdo               (LTC_ADC_SDO),                //                                   .sdo
		  
		  
			// motor (left motor exchange P/N for car)
        .dc_motor_left_conduit_end_motor_p     (MTRL_N),     //      dc_motor_left_conduit_end.motor_p
        .dc_motor_left_conduit_end_motor_n     (MTRL_P),     //                               .motor_n
        .dc_motor_right_conduit_end_motor_p    (MTRR_P),    //     dc_motor_right_conduit_end.motor_p
        .dc_motor_right_conduit_end_motor_n    (MTRR_N),     //                               .motor_n
		  
		  // light & buzzer
		  .light_left_external_connection_export     (LED_LEFT),     //     light_left_external_connection.export
        .light_right_external_connection_export    (LED_RIGHT),    //    light_right_external_connection.export
        .buzzer_external_connection_export         (buzzer_on),          //         buzzer_external_connection.export
		  
		  // IR
       .ir_rx_conduit_end_export               (IR_RXD),                //               ir_rx_conduit_end.export
		  
		  
		  .led_external_connection_export         (nios_led),          //         led_external_connection.export    
        .switch_external_connection_export      (SW),      //     switch_external_connection.export
		  
        .ir_led_on_external_connection_export         (IR_LED_ON),         //         ir_led_on_external_connection.export
        .scd_3v3_poweron_n_external_connection_export (VCC3P3_PWRON_n)  // scd_3v3_poweron_n_external_connection.export
		  
		  
	 );

////////////////////////////
// buzzer 	 
	 
// 1K for buzzer
reg [15:0]  cnt;
reg 			beep_level;
always @(posedge CLOCK_50)
begin
	if (cnt >= 16'd25000)
	begin
		cnt <= 0;
		beep_level <= ~beep_level;
	end
	else
		cnt <= cnt + 1;
end


assign BUZZER_OUT = buzzer_on?beep_level:1'bz;

////////////////////////////
// LED PWM 50% for power reduce
// led is high active
wire [7:0] nios_led;

assign LED[0] = nios_led[0]?led_pwm:1'b0;
assign LED[1] = nios_led[1]?led_pwm:1'b0;
assign LED[2] = nios_led[2]?led_pwm:1'b0;
assign LED[3] = nios_led[3]?led_pwm:1'b0;
assign LED[4] = nios_led[4]?led_pwm:1'b0;
assign LED[5] = nios_led[5]?led_pwm:1'b0;
assign LED[6] = nios_led[6]?led_pwm:1'b0;
assign LED[7] = nios_led[7]?led_pwm:1'b0;

reg led_pwm;
always @(posedge CLOCK_50)
begin
	led_pwm <= ~led_pwm;
end




endmodule
