/*
 * Motor.c
 *
 *  Created on: 2015/11/6
 *      Author: User
 */

#include "Motor.h"
#include <stdio.h>
#include <math.h>




#define REG_TOTAL_DUR	0
#define REG_HIGH_DUR	1
#define REG_CTS			2

#define RUN_BIT			0x01
#define FORWARD_BIT		0x02
#define FAST_DECAY_BIT	0x04


#define CYCLE_WIDTH 		1000  // scale x10
#define CYCLE_WIDTH_MAX 	CYCLE_WIDTH
#define CYCLE_WIDTH_MINI 	560   // 2.8V relative to 5V


#define MOTOR_MAX_VOL		6.0
#define MOTOR_MINI_VOL		2.8


CDcMotor::CDcMotor(int BaseAddress):
	m_BaseAddress(BaseAddress),
	m_CycleWidthMini(CYCLE_WIDTH_MINI),
	m_CycleWidthMaxi(CYCLE_WIDTH_MAX)

{
	// TODO Auto-generated constructor stub

//	m_mmap = new CMMap(CMMap::MAP_H2F_LW_BRIDGE);

	IOWR(m_BaseAddress, REG_TOTAL_DUR, CYCLE_WIDTH);
}

CDcMotor::~CDcMotor() {
	// TODO Auto-generated destructor stub
	//delete m_mmap;
}


void CDcMotor::SetDirection(bool bFordward){
	int Status,Control;
	Status = IORD(m_BaseAddress, REG_CTS);
	if (bFordward)
		Control = Status | FORWARD_BIT;
	else
		Control = Status & ~FORWARD_BIT;
	IOWR(m_BaseAddress, REG_CTS, Control);
}

void CDcMotor::SetSpeed(float fSpeed /* -100~100*/){
	int SpeedParam = 0;

	// bound speed
	if (fSpeed < -100.0)
		fSpeed = -100;
	else if (fSpeed > 100.0)
		fSpeed = 100.0;

	//
	if (fSpeed != 0.0){
		SpeedParam = m_CycleWidthMini + (int)(fabs(fSpeed) * (float)(m_CycleWidthMaxi-m_CycleWidthMini) / 100.0);
	}

	IOWR(m_BaseAddress, REG_HIGH_DUR, SpeedParam);
	SetDirection((fSpeed >= 0.0)?true:false);
}

void CDcMotor::Stop(void){
	int Status,Control;
	Status = IORD(m_BaseAddress, REG_CTS);
	Control = Status & (~RUN_BIT);
	IOWR(m_BaseAddress, REG_CTS, Control);
}

void CDcMotor::Start(void){
	int Status,Control;
	Status = IORD(m_BaseAddress, REG_CTS);
	Control = Status | RUN_BIT;
	IOWR(m_BaseAddress, REG_CTS, Control);

}

void CDcMotor::SetInputPower(alt_u16 mV){
	float fVol = (float)mV/1000.0;

	m_CycleWidthMini = (int)((float)MOTOR_MINI_VOL/fVol * (float)CYCLE_WIDTH);

	if (fVol < MOTOR_MAX_VOL)
		m_CycleWidthMaxi = CYCLE_WIDTH;
	else
		m_CycleWidthMaxi = (int)((float)MOTOR_MAX_VOL/fVol * (float)CYCLE_WIDTH);
}



