/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */

#include <stdio.h>
#include <stdio.h>
#include "terasic_includes.h"
#include "Car.h"
#include "Uart.h"
#include "altera_avalon_pio_regs.h"
#include "altera_avalon_uart_regs.h"


#define CAR_LED_ON  0x3
#define CAR_LED_OFF 0x0

typedef enum{
		CMD_SPEED,
	CMD_OFFSET,
	CMD_OFFSET_ADJ,
	CMD_STOP,
	CMD_START,
	CMD_LIGHT_ON,
	CMD_LIGHT_OFF,
	CMD_SPEAKER,
	CMD_VERSION,
	CMD_ASK_BATTERY,
	CMD_IDLE
}COMMAND_ID;

typedef struct{
	char szCommand[10];
	int CommandId;
	bool bParameter;
}COMMAND_INFO;

COMMAND_INFO gCommandList[] = {
		{"ATSP=", CMD_SPEED, true},
		{"ATOF=", CMD_OFFSET, true},
		{"ATOFA=", CMD_OFFSET_ADJ, true},
		{"ATSP\r", CMD_STOP, false},
		{"ATST\r", CMD_START, false},
		{"ATLON\r", CMD_LIGHT_ON, false},
		{"ATLOFF\r", CMD_LIGHT_OFF, false},
		{"ATSPKER\r", CMD_SPEAKER, false},
		{"ATVR\r", CMD_VERSION, false},
		{"ATAB\r", CMD_ASK_BATTERY, false}
};

bool CommandParsing(char *pCommand, int *pCommandID, int *pParam){
	bool bFind = false;
	int nNum, i, j, z=0,x=0;
	bool find_equal = false;
	char Data[10]={0};
	nNum = sizeof(gCommandList)/sizeof(gCommandList[0]);

	for(i=0;i<nNum && !bFind;i++){
		if (strncmp(pCommand, gCommandList[i].szCommand, strlen(gCommandList[i].szCommand)) == 0){
			*pCommandID = gCommandList[i].CommandId;
			if (gCommandList[i].bParameter){
				//*pParam = 10; //??
				for(j=0;pCommand[j]!='\r';j++){
					if(find_equal==true){
						Data[x] = pCommand[j];
						x++;
					}
					else if(pCommand[j]=='=')
						find_equal=true;
				}
				*pParam=atoi(Data);
			}
			bFind = true;
		} // if

	} // for
	return bFind;
}

bool CommandRead(CUart   &Uart, char *pBuffer, int nBufferSize, int *prx_lenmand){
	int RxLen;
	char Data;
	bool bDone = false;
	int Len = 0;
	alt_u32 timeout;

	if (!Uart.NoneBlockingRead((char *)&Data, sizeof(Data), &RxLen) || RxLen == 0)
			return false;

	*(pBuffer + 0) = Data;

	// read until find '\r'
	timeout = alt_nticks() + alt_ticks_per_second()/1000;
	Len = 1;
	while(!bDone){
		if (Uart.NoneBlockingRead((char *)&Data, sizeof(Data), &RxLen) && RxLen == sizeof(Data)){
			*(pBuffer + Len) = Data;
			Len++;
			if (Data == '\r')
				bDone = true;
		}else if (Len >= nBufferSize){
			// buffer full
			bDone = true;
		}else if (alt_nticks() > timeout){
			// timeout
			bDone = true;
		}else{
		}
		usleep(10);

	}
	*prx_lenmand = Len;
	return true;
}




void LightOn(bool bEnableLeft, bool bEnableRight){
	IOWR(CAR_LED_BASE, 0x00, (bEnableLeft?0x01:0x00)|(bEnableLeft?0x02:0x00));
}


void BuzzerOn(bool bEnable){
	IOWR(BUZZER_OUT_BASE, 0x00, bEnable?0x01:0x00);
}

void Welcome(CCar &Car){
	int i;
	const int nSpeed = 10;

	Car.Stop();
	Car.SetSpeed(nSpeed);

	for(i=0;i<2;i++){
		// FF
		Car.SetSpeed(nSpeed);
		Car.Start();


		// light on
		LightOn(true, true);

		// beep off
		BuzzerOn(false);

		usleep(50*1000);
		Car.Stop();

		///////////////////////

		Car.SetSpeed(-nSpeed);
		Car.Start();

		// light off
		LightOn(false, false);

		// beep on
		BuzzerOn(true);

		usleep(50*1000);
		Car.Stop();

	}
	// beep off
	BuzzerOn(false);

}



int main()
{
   CCar Car(DC_MOTOR_LEFT_BASE, DC_MOTOR_RIGHT_BASE);
   CUart Uart;
   alt_u8 Data8;
   int OffsetLevel=5;
   int nReadLen;
   int Command, Param;
   float RightMotor,LeftMotor;
   char szData[10];
   int Speed=0;
   char szADC_Data[10]={0};
   alt_u32 ADC_Data;
   alt_u32 LastActionTime;
   Welcome(Car);
   Car.Stop();
   Car.SetInputPower((alt_u16)IORD(CAR_VOLTAGE_DATA_BASE,0)*4);

   Uart.Open(BTS_TMD_NAME);

   printf("===== SmartCar =====\r\n");
   //IOWR(CAR_LED_BASE,0,0x3);
   while(1)
   {
		if(LastActionTime < alt_nticks())
		{
			LastActionTime = alt_nticks();
			IOWR(BUZZER_OUT_BASE,0,0);
		}

		if(CommandRead(Uart, szData, sizeof(szData), &nReadLen))
				{
					szData[nReadLen] = 0;

					if (CommandParsing(szData, &Command, &Param)){

					}else{
						printf("Invalid command\r\n");
					}
					switch(Command){
						case CMD_SPEED:
							Speed = Param;
							break;
						case CMD_OFFSET:
							LeftMotor = (float)Speed-(Param/OffsetLevel);
							RightMotor= (float)Speed+(Param/OffsetLevel);
							Car.SetSpeed(LeftMotor, RightMotor);

							break;
						case CMD_OFFSET_ADJ:
							OffsetLevel = 11-Param;
							break;
						case CMD_START:
							Car.Start();
							break;
						case CMD_STOP:
							Car.Stop();
							break;
						case CMD_LIGHT_ON:
							LightOn(true,true);
							break;
						case CMD_LIGHT_OFF:
							LightOn(false,false);
							break;
						case CMD_SPEAKER:
							BuzzerOn(true);
						    LastActionTime = alt_nticks()+200;
							break;
						case CMD_ASK_BATTERY:
							ADC_Data = IORD(CAR_VOLTAGE_DATA_BASE,0)*4;
							sprintf(szADC_Data,"%d\n",ADC_Data);
							Uart.WriteString(szADC_Data);
							break;

						default:printf("Nothing\n");
							break;
					}
				}
   }
  return 0;
}
